% This file contains the default parameterization for the elasticity
% routines of the Dynare toolbox.
% You may prefer not to modify this file. If you want to overwrite the
% default parameters for a model with results stored in MODELNAME.mat, 
% create a new file named MODELNAME_elast_parameters.m (or copy this file),
% and specify new parameter values in that file.
% The elasticity code attempts to run parameter files (if they exist) in
% the following sequence, overwriting older parameter values with newer
% ones:
% a) default_elast_parameters.m (in the toolbox directory)
% b) MODELNAME_elast_parameters.m
% c) MYPARAMFILE.m, in case that the elasticity calculation is called by
%        dynare_elasticity('MODELNAME','MYPARAMFILE');

%% ========================================================================
% directory from which to load models and where to store them
param.model_directory = 'models';
% directory where to save graphs
param.plot_savedirectory = 'graphs';

%% ========================================================================
% Model-related parameters

% period length at which the model is solved (e.g. 1/4 for quarterly)
param.model_periodlength = 1/4;
% period length unit name (e.g. 'quarters')
param.model_periodname = 'quarters';

% number of periods to calculate
param.model_T = 100;
% periods for which data is stored - make it sparser if you want to save
% memory (e.g., [0 : 10 : 100])
% param.model_Tgrid = [0 : param.model_T];

% type of recursive approximation method to be applied
% 1 - Split the dynamics of X into first- and second-order dynamics and
%     construct the dynamics for X1 and X2 (respects separation of first-
%     and second-order terms).
% 2 - Split the dynamics of X into first- and second-order dynamics and
%     construct the dynamics for X1 and X1+X2 (the X1+X2 variable is easier
%     to interpret as X).
param.model_recursive = 1;

%% ========================================================================
% Distribution simulation parameters
% The elasticity toolbox can simulate draws from the stationary
% distribution and then calculate quantiles of the stationary distribution
% of the shock elasticities.

% should distribution be simulated?
param.simul_distribution_run = 1;

% paths simulated in parallel
param.simul_paths = 200;
% number of initial periods dropped
param.simul_dropperiods = 200;
% number of periods used for the calculation of the distribution
%   total number of data points is simul_paths*simul_recordperiods
param.simul_recordperiods = 50;

% which statistics to calculate (0/1)
param.simul_calc_shockpriceelast = 1;     % shock-price elasticities
param.simul_calc_shockexposureelast = 1;  % shock-exposure elasticities
param.simul_calc_entropy_sdf = 0;         % entropy decomposition for SDFs
param.simul_calc_entropy_cf = 0;          % entropy decomposition for cash flows

% quantiles of the quantities to calculate
param.simul_quantiles = [0.25 0.5 0.75];

% threshold for resetting escaping paths in the simulations (set to zero if
% no threshold should be used). This may only be an issue for models which
% are not in the recursive-linear form.
param.simul_zerooutthreshold = 0;

%% ========================================================================
% plotting parameters

% plot any graphs - overrides subsequent setting for specific graphs
param.plot_graphs = 1;
param.plot_data_save = 1;
% plot specific graphs (0/1, default = 1)
param.plot_graphs_exposureelast = 1; % shock-price elasticities
param.plot_graphs_priceelast = 1;    % shock-exposure elasticities
param.plot_graphs_entropy_sdf = 0;   % entropy decomposition for SDFs
param.plot_graphs_entropy_cf = 0;    % entropy decomposition for cash flows

% period length for graph plotting (e.g. 1/4)
param.plot_periodlength = 1/4;
% period length unit name (e.g. 'quarters')
param.plot_periodname = 'quarters';
% dimension of the X axis
param.plot_horizon = 100;   % horizon, in units set in param.plot_periodlength

% scaling of the elasticity units (default = 1, annualized shock
% elasticity)
param.plot_elastscale = 1;
% shock elasticity unit description, plotted as the label on the vertical
% axis
param.plot_elastunit = '';

% plot elasticities evaluated at the deterministic steady state
param.plot_doxsteady = 1;
% plot elasticities evaluated at the point around which the expansion is
% expressed
param.plot_doxexp = 0;
% plot elasticities evaluated at Xh = XHBAR (fixed point of the approximate
% deterministic rule). If pruning is selected, this is the fixed point of
% the pruned solution.
param.plot_doxbar = 0;
% plot elasticities evaluated at Xh = XHMEAN (mean of the stationary
% distribution obtained from Dynare) - not recommended
param.plot_doxmean = 0;
% plot elasticities evaluated at quantiles of the stationary distribution
% (only if these have been calculated, see param.simul_calc_XXXX)
param.plot_doquantiles = 1;

% which shocks to plot - enumerate indices of shocks which should be
% plotted. If empty, then all shocks are plotted.
param.plot_shock_indices = [];

% saving options

% which graph formats should be saved
param.plot_saveeps = 1;
param.plot_savefig = 1;
param.plot_savepng = 0;
% distinguish filenames of the current plot variant
% graph filenames will start with "MODELNAME_param.plot_filename_spec"
param.plot_filename_spec = '';

% plot graph titles (0/1)
param.plot_graphtitle = 1;
% title note that will appear at the end of each graph title (for instance,
% when you want to plot different sets of graphs)
param.plot_titlenote = '';
% plot legend (0,1)
param.plot_graphlegend = 1;

% graph design options

% order of the line colors and styles in the graphs
param.plot_lineorder = {'b-','r--','m-.','k:','g-','c--','y-.','b:','r-','g--','k-.','m:','c-','y--','b-.'};
% graph line thickness 
param.plot_linewidth = 2;

% graph scaling (in pixels)
param.plot_horiz_scaling = 400;
param.plot_vert_scaling = 300;
% subplots per plot - default is 1 row, 1 column, but plots can be grouped
% in figures
% When rows = 1 and columns = 1, then every graph filename is identified by
% the identifier of the SDF and cash flow, respectively. When multiple
% graphs are plotted in one figure, then figures are numbered.
param.plot_figure_rows = 1;
param.plot_figure_columns = 1;

%% ========================================================================
% parameters that specify the behavior of the toolbox

% print detailed text output (0/1) - mostly technical details
param.env_displayverbose = 0;
% print no text output (0/1) (1 overrides env_displayverbose)
param.env_displaysilent = 0;