function elast = dynare_elasticity(varargin)
% [elast =] dynare_elasticity [(data_filename[, param_filename])]
%
% Calculates shock-price and shock-exposure elasticities, from a model
% calculated using Dynare or Dynare++.
%
% Input: If any input is omitted, it is assumed that dynare_elasticity is
%        called immediately after calling 'dynare model_filename', and
%        variables M_ and oo_ are stored in memory.
%        - data_filename - mat file with the model solution. If the mat
%                          file contains the oo_ variable, it is treated as
%                          a data file produced by Dynare, otherwise it is
%                          treated as a datafile produced by Dynare++.
%        - param_filename - filename with parameters that overrides default
%                           parameterization of the elasticity calculation.
%
% Output: elast - structure containing the results
%
% Examples: Start in the 'examples' directory, make sure that the
%           dynare elasticity toolbox folder (containing the function
%           dynare_elasticity.m) is on your path.
%
% 1) Solve the model example_RBC.mod using Dynare, calculate shock
%    elasticities from the model solution stored in memory, and store the
%    elasticities in the 'elast' structure.
%      dynare('example_RBC.mod');
%      elast = dynare_elasticity('');
%
% 2) Solve the model example_RBC.mod using Dynare, rename the model
%    solution file, calculate shock elasticities from the model solution
%    file, and store the elasticities in the 'elast' structure. Shock
%    elasticities will also be stored in the file 
%    'example_RBC_elasticity.mat'.
%        dynare('example_RBC.mod');
%        system('ren example_RBC_results.mat example_RBC.mat');
%          % on Unix machines, use mv instead of ren
%        elast = dynare_elasticity('example_RBC');
%
% 3) Solve the model example_RBC_plusplus.mod using Dynare++, calculate
%    shock elasticities from the model solution file, and store the shock
%    elasticities in the 'elast' structure. Shock elasticities will also be
%    stored in the file 'example_RBC_elasticity.mat'
%        system('dynare++ example_RBC_plusplus.mod');
%          % Dynare++ must be on your system path. Alternatively, run
%          % Dynare++ from the command line.
%        elast = dynare_elasticity('example_RBC_plusplus');
%
% 4) Calculate shock elasticities from a model solution stored in
%    'example_RBC.mat', using a parameter file name 'myparam.m' which
%    overrides default parameter values. The parameter files are searched
%    in the following order:
%    a) 'default_elast_parameters.m' - stored in the toolbox directory,
%       searched automatically
%    b) 'modelname_elast_parameters.m' - stored in the directory where
%       the model solution file is. In this example, the toolbox would
%       search for 'examples/example_RBC_elast_parameters.m'. This file
%       does not need to exist.
%    c) 'myparamfile.m' - if it is specified as the second argument
%       when running dynare_elasticity.
%       This file should be stored in the same directory as the model
%       solution file (in this example, 'examples/myparamfile.m').
%    Example code:
%        elast = dynare_elasticity('example_RBC','myparamfile');

tic;

% =========================================================================
% model file name
if ((isempty(varargin)) || (strcmp(cell2mat(varargin(1)),'')))
    param.data_filename = '';
else
    param.data_filename = cell2mat(varargin(1));
end

param.model_name = param.data_filename;

% parameter file name
if (length(varargin) >= 2)
    param.parameter_filename = cell2mat(varargin(2));
else
    param.parameter_filename = '';
end

% load parameters (and also display initial message)
disp_init_message = 1;
load_parameters;
clear disp_init_message;

% =========================================================================
% load model solution data, and process it

if (isempty(param.data_filename))
    % No filename provided, assume that model solution from Dynare is
    % stored in memory.

    disp_reg(param,0,'Loading model solution from memory.');
    global oo_ M_ options_;
    if ((~isstruct(oo_)) || (~isstruct(M_)) || (~isstruct(options_)))
        error('Required variables (oo_, M_, options_) missing, and no data file provided.');
    end
    dynres.oo_ = oo_;
    dynres.M_ = M_;
    dynres.options_ = options_;
    
    model_original = processfile_dynare(dynres,param);
    
else
    % Filename provided - load it, and identify whether it is a Dynare or
    % Dynare++ model solution file, based on the existence of oo_.
    
    if (exist(param.data_filename,'file'))
        dynres = load(param.data_filename,'-mat');
    elseif (exist(sprintf('%s.mat',param.data_filename),'file'))
        dynres = load(param.data_filename,'-mat');
    else
        error(sprintf('File %s(.mat) does not exist.', param.data_filename));
    end
    
    if (isfield(dynres,'oo_'))
        % Model solution created by Dynare.
        disp_reg(param,0,sprintf('Loading Dynare model solution file %s.', param.data_filename));
        model_original = processfile_dynare(dynres,param);
    elseif (isfield(dynres,'dyn_vars'))
        % Model solution created by Dynare++.
        disp_reg(param,0,sprintf('Loading Dynare++ model solution file %s.', param.data_filename));
        model_original = processfile_dynareplusplus(dynres,param);
    else
        error('Model solution file does not seem to be produced by Dynare or Dynare++.');
    end
    
end

% =========================================================================
% Required data at this point
%   X - complete law of motion
%   structure of variables from which I can extract sdf and cf names
% Start processing.


model = model_original;

X = model.X;
Esdf = model.Esdf;
sdf_name = model.sdf_name;
sdf_index = model.sdf_index;
Ecf = model.Ecf;
cf_name = model.cf_name;
cf_index = model.cf_index;

% =========================================================================
% Precalculate kronecker products and sums for increased speed.
% These will not be stored in the output file.
X.kTHETA10THETA10 = kron(X.THETA10,X.THETA10);
X.ksumTHETA10THETA11 = kron(X.THETA10,X.THETA11)+kron(X.THETA11,X.THETA10);
X.kTHETA11THETA11 = kron(X.THETA11,X.THETA11);
X.ksumTHETA10LAMBDA10 = kron(X.THETA10,X.LAMBDA10)+kron(X.LAMBDA10,X.THETA10);
nn = size(X.THETA11,2);
kk = size(X.LAMBDA10,2);
kronLAMBDA10THETA11 = zeros(nn*nn,nn*kk);
for i = 1 : nn
    kronLAMBDA10THETA11(:,(i-1)*kk+1:i*kk) = kron(X.LAMBDA10,X.THETA11(:,i));
end
X.ksumTHETA11LAMBDA10 = kron(X.THETA11,X.LAMBDA10)+kronLAMBDA10THETA11;
X.kLAMBDA10LAMBDA10 = kron(X.LAMBDA10,X.LAMBDA10);


% =========================================================================
% calculate shock elasticities and entropy decompositions

disp_reg(param,1,sprintf('==================================================================='));
disp_reg(param,0,sprintf('Calculating shock-exposure elasticities (%5.3f seconds).',toc));

param.type = 'exposure';
for j = 1 : length(cf_index)
    param.cf_name = cf_name(j);
    param.cf_index = cf_index(j);
    param.sdf_name = [];
    param.sdf_index = [];
    disp_reg(param,1,sprintf('Calculating shock-exposure elasticity for %s (Eexposure(%d)).', ...
        cell2mat(cf_name(j)), j));
    Eexposure(j) = construct_shock_elasticities(X,Ecf(j),param);
    
end

disp_reg(param,0,sprintf('Calculating shock-price elasticities (%5.3f seconds).',toc));

length(model.sdf_index)
length(model.cf_index)

for j = 1 : length(sdf_index)
    for k = 1 : length(cf_index)

        M.GAMMA0 = Esdf(j).GAMMA0 + Ecf(k).GAMMA0;
        M.GAMMA1 = Esdf(j).GAMMA1 + Ecf(k).GAMMA1;
        M.GAMMA2 = Esdf(j).GAMMA2 + Ecf(k).GAMMA2;
        M.GAMMA3 = Esdf(j).GAMMA3 + Ecf(k).GAMMA3;
        M.PSI0 = Esdf(j).PSI0 + Ecf(k).PSI0;
        M.PSI1 = Esdf(j).PSI1 + Ecf(k).PSI1;
        M.PSI2 = Esdf(j).PSI2 + Ecf(k).PSI2;
        
        param.cf_name = cf_name(k);
        param.cf_index = cf_index(k);
        param.sdf_name = sdf_name(j);
        param.sdf_index = sdf_index(j);
        
        param.type = 'price';
        disp_reg(param,1,sprintf('Calculating shock-price elasticity for %s and %s (Eprice(%d,%d)).', ...
            cell2mat(sdf_name(j)), cell2mat(cf_name(k)), j, k));
        Eprice(j,k) = construct_shock_elasticities(X,M,param,Eexposure(k));
        
    end
end

disp_reg(param,0,sprintf('Calculating entropy measures (%5.3f seconds).',toc));

param.type = 'entropy_cf';
for j = 1 : length(cf_index)
    param.cf_name = cf_name(j);
    param.cf_index = cf_index(j);
    param.sdf_name = [];
    param.sdf_index = [];
    disp_reg(param,1,sprintf('Calculating entropy for %s (Eentropy_cf(%d)).', ...
        cell2mat(cf_name(j)), j));
    Eentropy_cf(j) = construct_entropy(X,Ecf(j),param);
end

param.type = 'entropy_sdf';
for j = 1 : length(sdf_index)
    param.cf_name = [];
    param.cf_index = [];
    param.sdf_name = sdf_name(j);
    param.sdf_index = sdf_index(j);
    disp_reg(param,1,sprintf('Calculating entropy for %s (Eentropy_sdf(%d)).', ...
        cell2mat(sdf_name(j)), j));
    Eentropy_sdf(j) = construct_entropy(X,Esdf(j),param);
end


% =========================================================================
% simulate stationary distribution and calculate elasticities under the
% simulated distribution

clear object_input;

if (param.simul_distribution_run)
    if (param.simul_calc_shockpriceelast)
        object_input.Eprice = Eprice;
    end
    if (param.simul_calc_shockexposureelast)
        object_input.Eexposure = Eexposure;
    end
    if (param.simul_calc_entropy_sdf)
        object_input.Eentropy_sdf = Eentropy_sdf;
    end
    if (param.simul_calc_entropy_cf)
        object_input.Eentropy_cf = Eentropy_cf;
    end

    if (exist('object_input','var'))
        disp_reg(param,1,sprintf('==================================================================='));
        disp_reg(param,0,sprintf('Calculating quantiles of elasticity distributions (%5.3f seconds).', toc));
        objects = simulate_distribution(X,param,object_input);
    end

    if (param.simul_calc_shockpriceelast)
        Eprice = objects.Eprice;
    end
    if (param.simul_calc_shockexposureelast)
        Eexposure = objects.Eexposure;
    end
    if (param.simul_calc_entropy_sdf)
        Eentropy_sdf = objects.Eentropy_sdf;
    end
    if (param.simul_calc_entropy_cf)
        Eentropy_cf = objects.Eentropy_cf;
    end
end

clear object_input objects;

% =========================================================================
% store calculated results

elast.param = param;
elast.model_original = model_original;
elast.model = model;
elast.X = model.X; % omit added auxiliary variables like Kronecker products
elast.Ecf = Ecf;
elast.Esdf = Esdf;
elast.Eexposure = Eexposure;
elast.Eprice = Eprice;
elast.Eentropy_cf = Eentropy_cf;
elast.Eentropy_sdf = Eentropy_sdf;

if (~isempty(param.data_filename))
    param.output_filename = sprintf('%s_elasticity',param.data_filename);
    save(sprintf('%s.mat',param.output_filename),'elast')
    disp_reg(param,0,sprintf('Results saved in %s.mat.',param.output_filename));
else
    param.output_filename = '';
end

% =========================================================================
% plot graphs

if (param.plot_graphs)
    disp_reg(param,1,sprintf('==================================================================='));
    disp_reg(param,0,sprintf('Plotting graphs (%5.3f seconds).',toc));
    plot_elast_graphs(elast,0);
end

disp_reg(param,0,sprintf('Calculation finished in %5.3f seconds.',toc));

end
