function fig_handles = plot_elast_graphs(model, param_filename)
% fig_handles = plot_elast_graphs(model [, param_filename])
% Plots all requested graphs, either from a supplied data structure, or
% from a model file.
% 
% Input: model - a filename with stored shock elasticities, or an
%                elasticity data structure
%        param_filename - a specific parameter file that overrides settings
%                         for graph plotting
%
% Examples: start with a model solution file modelname.mat.
% 1) Using a data structure previously obtained from dynare_elasticity.
%      elast = dynare_elasticity('modelname');
%      figs = plot_elast_graphs(elast);
% 2) Using a stored data structure, and calling parameters from
%    'plotparamfile.mat'.
%      elast = dynare_elasticity('modelname');
%      figs = plot_elast_graphs('modelname_elasticity','plotparamfile');

%% ========================================================================
% input argument can be either model structure, or filename with the
% results.
if (isstruct(model))
    elast = model;
else
    load(sprintf('%s.mat',model));
end

% distribute model structure
fn = fieldnames(elast);
for i = 1 : length(fn)
    eval(sprintf('%s = elast.(cell2mat(fn(i)));',cell2mat(fn(i))));
end

% load a new parameter file name if the second parameter is set
if ((exist('param_filename','var')) && (exist(sprintf('%s.m',param_filename),'file')))
    eval (sprintf('%s', param_filename));
    disp_reg(param,0,sprintf('Parameter file %s.m loaded.',param_filename));
end

% prepare shock legend
param.leg_shock_names = {};
for i = 1 : size(model.shock_names)
    param.leg_shock_names(i) = {strrep(model.shock_names(i,:), '_','\_')};
end

% identify indices to be plotted
if (isempty(param.plot_shock_indices))
    param.plot_shock_indices = [1:size(elast.Eexposure(1).GAMMA0,1)];
end

% check whether title note is present
if (~isempty(param.plot_titlenote))
    param.plot_titlenote = sprintf(' (%s)', param.plot_titlenote);
end

% set up graph directory
if (isempty(param.plot_savedirectory))
    param.dirprefix = '';
else
    if (~exist(param.plot_savedirectory,'dir'))
        [status,result] = system(sprintf('mkdir %s',param.plot_savedirectory'));
        if (status)
            error('Failed to create directory %s. Error message:\n%s', param.plot_savedirectory, result);
        end
    end
    param.dirprefix = strcat(param.plot_savedirectory,'/');
end

% set up graph filename
if (~strcmp(param.plot_filename_spec,''))
    param.graph_savefilename = sprintf('%s_%s_', param.model_name, param.plot_filename_spec);
else
    param.graph_savefilename = sprintf('%s_', param.model_name);
end
while ((~isempty(param.graph_savefilename)) && (strcmp(param.graph_savefilename(1),'_')))
    param.graph_savefilename = param.graph_savefilename(2:end);
end

% set graph scaling
scrs = get (0, 'ScreenSize');
horiz_scaling = param.plot_horiz_scaling / scrs(3);
vert_scaling = param.plot_vert_scaling / scrs(4);

fig_scaling = [ (1-horiz_scaling)/2*scrs(3)
                (1-vert_scaling)/2*scrs(4)
                scrs(3)*horiz_scaling
                scrs(4)*vert_scaling ];

fig_handles = [];

%% ========================================================================
% shock-price elasticities
if (param.plot_graphs_priceelast)

for j = 1 : size(Eprice,1)
    graph_id = 0;
    for k = 1 : size(Eprice,2)
        
        if (mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0)
            fig = figure;
            fig_handles(length(fig_handles)+1) = fig;
            set (fig, 'PaperPositionMode', 'auto');
            set (fig, 'Position', fig_scaling);

        end
        graph_id = graph_id + 1;
        if (param.plot_figure_rows * param.plot_figure_columns > 1)
            subplot(param.plot_figure_rows,param.plot_figure_columns, ...
                mod(graph_id-1,param.plot_figure_rows*param.plot_figure_columns)+1);
        end
        
        Eprice(j,k).name = [cell2mat(Eprice(j,k).sdf_name) ' - ' cell2mat(Eprice(j,k).cf_name)];
        data = plot_elast_plotgraph(param,Eprice(j,k));
               
        if ((mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0) || (k == size(Eprice,2)))
            % save graphs
            if (param.plot_figure_rows*param.plot_figure_columns == 1)
                % if figures contain individual graphs, save under individual
                % names
                plot_elast_savegraph(param,fig,sprintf('%spriceelast_%s_%s', ...
                    param.graph_savefilename, cell2mat(Eprice(j,k).sdf_name), cell2mat(Eprice(j,k).cf_name)));
            else
                % otherwise just index by number
                plot_elast_savegraph(param,fig,sprintf('%spriceelast_%d', ...
                    param.graph_savefilename, ceil(graph_id/(param.plot_figure_rows*param.plot_figure_columns))));
            end            
        end
        if (param.plot_data_save == 1)
            % store graph data for replotting
            fileID = fopen(sprintf('%s%spriceelast_%s_%s_data.dat', ...
                param.dirprefix,param.graph_savefilename, cell2mat(Eprice(j,k).sdf_name), ...
                cell2mat(Eprice(j,k).cf_name)),'w');
            fprintf(fileID,sprintf('%s\n',data.header));
            for ii = 1 : length(param.plot_shock_indices)
                iii = param.plot_shock_indices(ii);
                fprintf(fileID,sprintf('# shock s%d - %s\n',iii,elast.model.shock_names(iii,:)));                
            end
            fprintf(fileID,sprintf(strcat(data.format,'\n'),data.content'));
            fclose(fileID);
        end
        
    end
end
end

%% =========================================================================
% shock-exposure elasticities
if (param.plot_graphs_exposureelast)
graph_id = 0;

for j = 1 : length(Eexposure)
    
    if (mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0)
        fig = figure;
        fig_handles(length(fig_handles)+1) = fig;
        set (fig, 'PaperPositionMode', 'auto');
        set (fig, 'Position', fig_scaling);
    end
    graph_id = graph_id + 1;
    if (param.plot_figure_rows * param.plot_figure_columns > 1)
        subplot(param.plot_figure_rows,param.plot_figure_columns, ...
            mod(graph_id-1,param.plot_figure_rows*param.plot_figure_columns)+1);
    end

    Eexposure(j).name = cell2mat(Eexposure(j).cf_name);
    data = plot_elast_plotgraph(param,Eexposure(j));

    if ((mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0) || (j == length(Eexposure)))
        % save graphs
        if (param.plot_figure_rows*param.plot_figure_columns == 1)
            % if figures contain individual graphs, save under individual
            % names
            plot_elast_savegraph(param,fig, sprintf('%sexposureelast_%s', ...
                param.graph_savefilename, cell2mat(Eexposure(j).cf_name)));
        else
            % otherwise just index by number
            plot_elast_savegraph(param,fig,sprintf('%sexposureelast_%d', ...
                param.graph_savefilename, ceil(graph_id/(param.plot_figure_rows*param.plot_figure_columns))));
        end
    end 
    
    if (param.plot_data_save == 1)
        % store graph data for replotting
        fileID = fopen(sprintf('%s%sexposureelast_%s_data.dat', ...
                param.dirprefix,param.graph_savefilename, ...
                cell2mat(Eexposure(j).cf_name)),'w');
        fprintf(fileID,sprintf('%s\n',data.header));
        for ii = 1 : length(param.plot_shock_indices)
            iii = param.plot_shock_indices(ii);
            fprintf(fileID,sprintf('# shock s%d - %s\n',iii,elast.model.shock_names(iii,:)));                
        end
        fprintf(fileID,sprintf(strcat(data.format,'\n'),data.content'));
        fclose(fileID);
    end
    
end
end

%% =========================================================================
% entropy decomposition for stochastic discount factors
if (param.plot_graphs_entropy_sdf)
graph_id = 0;
plot_shock_indices = param.plot_shock_indices;
param.plot_shock_indices = 1;

for j = 1 : length(Eentropy_sdf)
    
    if (mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0)
        fig = figure;
        fig_handles(length(fig_handles)+1) = fig;
        set (fig, 'PaperPositionMode', 'auto');
        set (fig, 'Position', fig_scaling);
    end
    graph_id = graph_id + 1;
    if (param.plot_figure_rows * param.plot_figure_columns > 1)
        subplot(param.plot_figure_rows,param.plot_figure_columns, ...
            mod(graph_id-1,param.plot_figure_rows*param.plot_figure_columns)+1);
    end

    Eentropy_sdf(j).name = sprintf('entropy - %s',cell2mat(Eentropy_sdf(j).sdf_name));
    data = plot_elast_plotgraph(param,Eentropy_sdf(j));

    if ((mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0) || (j == length(Eentropy_sdf)))
        % save graphs
        if (param.plot_figure_rows*param.plot_figure_columns == 1)
            % if figures contain individual graphs, save under individual
            % names
            plot_elast_savegraph(param,fig, sprintf('%sentropy_%s', ...
                param.graph_savefilename, cell2mat(Eentropy_sdf(j).sdf_name)));
        else
            % otherwise just index by number
            plot_elast_savegraph(param,fig,sprintf('%sentropy_sdf_%d', ...
                param.graph_savefilename, ceil(graph_id/(param.plot_figure_rows*param.plot_figure_columns))));
        end
    end 
end

param.plot_shock_indices = plot_shock_indices;
end

%% =========================================================================
% entropy decomposition for cash flows
if (param.plot_graphs_entropy_cf)
graph_id = 0;
plot_shock_indices = param.plot_shock_indices;
param.plot_shock_indices = 1;

for j = 1 : length(Eentropy_cf)
    
    if (mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0)
        fig = figure;
        fig_handles(length(fig_handles)+1) = fig;
        set (fig, 'PaperPositionMode', 'auto');
        set (fig, 'Position', fig_scaling);
    end
    graph_id = graph_id + 1;
    if (param.plot_figure_rows * param.plot_figure_columns > 1)
        subplot(param.plot_figure_rows,param.plot_figure_columns, ...
            mod(graph_id-1,param.plot_figure_rows*param.plot_figure_columns)+1);
    end

    Eentropy_cf(j).name = sprintf('entropy - %s',cell2mat(Eentropy_cf(j).cf_name));
    data = plot_elast_plotgraph(param,Eentropy_cf(j));

    if ((mod(graph_id,param.plot_figure_rows*param.plot_figure_columns) == 0) || (j == length(Eentropy_cf)))
        % save graphs
        if (param.plot_figure_rows*param.plot_figure_columns == 1)
            % if figures contain individual graphs, save under individual
            % names
            plot_elast_savegraph(param,fig, sprintf('%sentropy_%s', ...
                param.graph_savefilename, cell2mat(Eentropy_cf(j).cf_name)));
        else
            % otherwise just index by number
            plot_elast_savegraph(param,fig,sprintf('%sentropy_cf_%d', ...
                param.graph_savefilename, ceil(graph_id/(param.plot_figure_rows*param.plot_figure_columns))));
        end
    end 
end

param.plot_shock_indices = plot_shock_indices;
end

end