function model = processfile_dynare(dynres, param)
% Extracts common variables from a Dynare model solution file.

% =========================================================================
%  variable names:
%  The correct order of the variables in declaration order is
%  stored in M_.endo_names, not in var_list_ !!!
%    M_.endo_names stores variables in declaration order
%    var_list_declaration is just M_.endo_names
%    var_list_reordered stores variables in reordered order
%    var_list_declaration(i) is the same as
%      var_list_reordered(oo_.dr.inv_order_var(i))
%    var_list_reordered(i) is the same as
%      var_list_declaration(oo_.dr.order_var(i))
% 
% oo_ contains:
%  dr - coefficients on the law of motion
%  dr.ys - steady state
% 
% =========================================================================

oo_ = dynres.oo_;
M_ = dynres.M_;
options_ = dynres.options_;

% =========================================================================
% names of variables
model.var_names = M_.endo_names(oo_.dr.order_var,:);
model.var_names_tex = M_.endo_names_tex(oo_.dr.order_var,:);
model.shock_names = M_.exo_names;
model.shock_names_tex = M_.exo_names_tex;

% indices of state variables
model.state_vars = [M_.nstatic+1:M_.nstatic+ size(oo_.dr.ghx,2)];

% count variables, states, and shocks
m = size(oo_.dr.ghx,1);      % number of all variables
n = size(oo_.dr.ghx,2);      % length of the actual state vector
k = size(oo_.dr.ghu,2);      % length of the shock vector

% =========================================================================
% Law of motion for the state vector stored as
% X(t+1) = A0 + A1*(X(t),W(t+1)) + A2*kron(X(t),W(t+1),X(t),W(t+1)) + ...
% or, alternatively, as
% X(t+1) = A0 + AX*X(t) + AW*W(t+1) + AXX*kron(X(t),X(t)) + ...
%          2*AXW*kron(X(t),W(t+1)) + AWW*kron(W(t+1),W(t+1)) + ...
% All coefficient matrices stored in X.
% The goal is to produce a representation equivalent to the one produced by
% Dynare:
% X(t+1) = KK + AA*X(t) + BB*W(t+1) + 1/2*CC*kron(X(t),X(t))
%          + 1/2*DD*kron(W(t+1),W(t+1)) + FF*kron(X(t),W(t+1))
% Notice the difference in scaling the coefficients on order j by j!.

X.AX = oo_.dr.ghx;
X.AW = oo_.dr.ghu;

if (isfield(oo_.dr,'ghxx'))
    X.A0 = oo_.dr.ghs2 / 2;
    X.AXX = oo_.dr.ghxx / 2;
    X.AXW = oo_.dr.ghxu / 2;
    X.AWW = oo_.dr.ghuu / 2;
    model.expansion_order = 2;
else
    % if expansion order is only 1, generate zero second-order matrices
    X.A0 = zeros(m,1);
    X.AXX = zeros(m,n*n);
    X.AXW = zeros(m,n*k);
    X.AWW = zeros(m,k*k);
    model.expansion_order = 1;
end

X.A1 = [X.AX X.AW];
% reconstruct the second derivative matrix A2
X.A2 = zeros(m,(n+k)^2);
for i = 1 : n
    X.A2(:,(i-1)*(n+k)+1:i*(n+k)) = [X.AXX(:,(i-1)*n+1:i*n) X.AXW(:,(i-1)*k+1:i*k)];
end
for i = 1 : k
    for j = 1 : n
        X.A2(:,(n+i-1)*(n+k)+j) = X.AXW(:,(j-1)*k+i);
    end
    X.A2(:,(n+i-1)*(n+k)+n+1:(n+i)*(n+k)) = X.AWW(:,(i-1)*k+1:i*k);
end

% deterministic steady state
X.XSTEADY = oo_.dr.ys(oo_.dr.order_var); % steady state is not reordered originally
% point of expansion (i.e. the point relative to which the expansion is
% expressed). This is the deterministic steady state when --no-centralize
% is used, or the fix point of the rule without the --no-centralize rule.
X.XEXP = X.XSTEADY;
% mean of the stationary distribution
X.XMEAN = oo_.mean(oo_.dr.order_var);
% variance matrix of the shocks
X.SIGMA2 = M_.Sigma_e;

model.X = X;

model = processfile_dynare_step2(model,param);

end