function model = processfile_dynare_step2(model_original,param)

% =========================================================================
% apply pruning
% model_original = model;
% if (model.expansion_order >= 2)
model = recursive_dynamics(model_original,param);
% is_pruned = 1;
% else
%     is_pruned = 0;
% end

% =========================================================================
% Law of motion for the state vector, under the recursively-linear
% dynamics, written using Dynare++ notation. X is in deviations from the
% expansion point (deterministic steady state or fix point).
% X(t+1) = KK + AA*X(t) + BB*W(t+1) + 1/2*CC*kron(X(t),X(t))
%          + 1/2*DD*kron(W(t+1),W(t+1)) + FF*kron(X(t),W(t+1))

m = size(model.X.AX,1); % number of all variables
n = size(model.X.AX,2); % length of the actual state vector
k = size(model.X.AW,2); % length of the shock vector

% trim the state dynamics only to the state variables (original dynamics)
X.AX = model.X.AX(model.state_vars,:);
X.AW = model.X.AW(model.state_vars,:);
X.AXX = model.X.AXX(model.state_vars,:);
X.AWW = model.X.AWW(model.state_vars,:);
X.AXW = model.X.AXW(model.state_vars,:);
X.A0 = model.X.A0(model.state_vars,:);

% =========================================================================
% prepare variables for the triangular structure obtained from pruning
% X1' = THETA10 + THETA11 X1 + LAMBDA10 W
% X2' = THETA20 + THETA21 X1 + THETA22 X2 + THETA23 kron(X1,X1) +
%       LAMBDA20 W + LAMBDA21 kron(X1,W) + LAMBDA22 kron(W,W)

X.THETA10 = 0*model_original.X.A0(model_original.state_vars,:);
X.THETA11 = model_original.X.AX(model_original.state_vars,:);
X.LAMBDA10 = model_original.X.AW(model_original.state_vars,:);
X.THETA20 = model_original.X.A0(model_original.state_vars,:);
X.THETA21 = 0*model_original.X.AX(model_original.state_vars,:);
X.THETA22 = model_original.X.AX(model_original.state_vars,:);
X.THETA23 = model_original.X.AXX(model_original.state_vars,:);
X.LAMBDA20 = 0*model_original.X.AW(model_original.state_vars,:);
X.LAMBDA21 = 2*model_original.X.AXW(model_original.state_vars,:);
X.LAMBDA22 = model_original.X.AWW(model_original.state_vars,:);

% =========================================================================
% calculate interesting points in the dynamics

% XEXP - Expansion point - the point relative to which the dynamics is
%        expressed. In Dynare, this is the deterministic steady state
%        (i.e., the expansion point).
%        In Dynare++, this can be the deterministic steady state (expansion
%        point), or the fix point around which the rule is re-centered.
X.XEXP = model.X.XEXP(model.state_vars,:);
% XSTEADY - steady state, absolute
X.XSTEADY = model.X.XSTEADY(model.state_vars,:);
% XMEAN - mean of the stationary distribution. Simulated
%         by Dynare and Dynare++, and typically differs slightly between
%         the two.
X.XMEAN = model.X.XMEAN(model.state_vars,:);

% XHSTEADY - steady state, relative to XEXP
X.XHSTEADY = X.XSTEADY - X.XEXP;
% XHMEAN - Mean of the stationary distribution, relative to XEXP. Simulated
%          by Dynare and Dynare++.
X.XHMEAN = X.XMEAN - X.XEXP;
% XHBAR - Fixed point, relative to XEXP. The point that satisfies
%         XHBAR = g(XHBAR,0) in the approximate decision rule.
X.XHBAR = calculate_fix_point(X, param);

% decompose the variance matrix
X.SIGMA = chol(model.X.SIGMA2,'lower');
if (sum(sum(diag(diag(model.X.SIGMA2))~=model.X.SIGMA2)) > 0)
    warning('Shocks are not orthogonal, this is required!')
end

Xmodel = X;

% =========================================================================
% detect all stochastic discount factors and cash-flow processes

% exclude variables added during pruning
nonpruned_ind = [1:n/2 n+1:m];

% find the variable called sdf..., and stores its index to sdf_index
disp_reg(param,1,sprintf('==================================================================='));
sdf_index = [];
sdf_name = {};

for jj = 1 : length(nonpruned_ind)
    j = nonpruned_ind(jj);
    if (strncmp('sdf',strtrim(model.var_names(j,:)),3))
        sdf_index(length(sdf_index)+1) = j;
        sdf_name(length(sdf_index)) = {strtrim(model.var_names(j,:))};
    end
end
disp_reg(param,1,sprintf('''sdf'' variables found: %d', length(sdf_index)));
for j = 1 : length(sdf_index)
    disp_reg(param,1,sprintf('    %s', cell2mat(sdf_name(j))));
end

% find the variable called cf..., and stores its index to sdf_index
cf_index = [];
cf_name = {};
for jj = 1 : length(nonpruned_ind)
    j = nonpruned_ind(jj);
    if (strncmp('cf',strtrim(model.var_names(j,:)),2))
        cf_index(length(cf_index)+1) = j;
        cf_name(length(cf_index)) = {strtrim(model.var_names(j,:))};
    end
end
disp_reg(param,1,sprintf('''cf'' variables found: %d', length(cf_index)));
for j = 1 : length(cf_index)
    disp_reg(param,1,sprintf('    %s', cell2mat(cf_name(j))));
end

% =========================================================================
% construct SDFs and cash flows

for j = 1 : length(sdf_index)
%     Esdf(j).XEXP = model.X.XEXP(sdf_index(j));
%     Esdf(j).XHBAR = X.XHBARFULL(sdf_index(j));
%     Esdf(j).XHMEAN = X.XHMEANFULL(sdf_index(j));
    Esdf(j).GAMMA0 = model_original.X.XEXP(model.var_index_orig(sdf_index(j)),:) ...
                     + model_original.X.A0(model.var_index_orig(sdf_index(j)),:);
    Esdf(j).GAMMA1 = model_original.X.AX(model.var_index_orig(sdf_index(j)),:);
    Esdf(j).GAMMA2 = model_original.X.AX(model.var_index_orig(sdf_index(j)),:);
    Esdf(j).GAMMA3 = model_original.X.AXX(model.var_index_orig(sdf_index(j)),:);
    Esdf(j).PSI0 = model_original.X.AW(model.var_index_orig(sdf_index(j)),:);
    Esdf(j).PSI1 = 2*model_original.X.AXW(model.var_index_orig(sdf_index(j)),:);
    Esdf(j).PSI2 = model_original.X.AWW(model.var_index_orig(sdf_index(j)),:);
end

for j = 1 : length(cf_index)
%     Ecf(j).XEXP = model.X.XEXP(cf_index(j));
%     Ecf(j).XHBAR = X.XHBARFULL(cf_index(j));
%     Ecf(j).XHMEAN = X.XHMEANFULL(cf_index(j));
    Ecf(j).GAMMA0 = model_original.X.XEXP(model.var_index_orig(cf_index(j)),:) ...
                     + model_original.X.A0(model.var_index_orig(cf_index(j)),:);
    Ecf(j).GAMMA1 = model_original.X.AX(model.var_index_orig(cf_index(j)),:);
    Ecf(j).GAMMA2 = model_original.X.AX(model.var_index_orig(cf_index(j)),:);
    Ecf(j).GAMMA3 = model_original.X.AXX(model.var_index_orig(cf_index(j)),:);
    Ecf(j).PSI0 = model_original.X.AW(model.var_index_orig(cf_index(j)),:);
    Ecf(j).PSI1 = 2*model_original.X.AXW(model.var_index_orig(cf_index(j)),:);
    Ecf(j).PSI2 = model_original.X.AWW(model.var_index_orig(cf_index(j)),:);
end

model.X = X;
model.Esdf = Esdf;
model.sdf_name = sdf_name;
model.sdf_index = sdf_index;
model.Ecf = Ecf;
model.cf_name = cf_name;
model.cf_index = cf_index;

end