function model = processfile_dynareplusplus(dynres,param)
% Extracts common variables from a Dynare++ model solution file.

% =========================================================================
% names of variables
model.var_names = dynres.dyn_vars;
model.var_names_tex = dynres.dyn_vars;
model.shock_names = dynres.dyn_shocks;
model.shock_names_tex = dynres.dyn_shocks;

% indices of state variables
model.state_vars = [dynres.dyn_nstat+1 : dynres.dyn_nstat + dynres.dyn_npred + dynres.dyn_nboth];

% count variables, states, and shocks
m = size(dynres.dyn_vars,1);       % number of all variables
n = length(model.state_vars);      % length of the actual state vector
k = size(dynres.dyn_shocks,1);     % length of the shock vector

% =========================================================================
% Law of motion for the state vector stored as
% X(t+1) = A0 + A1*(X(t),W(t+1)) + A2*kron(X(t),W(t+1),X(t),W(t+1)) + ...
% or, alternatively, as
% X(t+1) = A0 + AX*X(t) + AW*W(t+1) + AXX*kron(X(t),X(t)) + ...
%          2*AXW*kron(X(t),W(t+1)) + AWW*kron(W(t+1),W(t+1)) + ...
% All coefficient matrices stored in X.
% The goal is to produce a representation equivalent to the one produced by
% Dynare:
% X(t+1) = KK + AA*X(t) + BB*W(t+1) + 1/2*CC*kron(X(t),X(t))
%          + 1/2*DD*kron(W(t+1),W(t+1)) + FF*kron(X(t),W(t+1))
% Notice the difference in scaling the coefficients on order j by j!.

if (size(dynres.dyn_steady_states,2) > 1)
    disp_reg(param,1,'Check that the multistep procedure is implemented correctly.');
end

% now iterate on the dimensions and construct the coefficient matrices
model.expansion_order = 0;
iter_finish = 0;

% constant term in the law of motion
if (isfield(dynres,'dyn_g_0'))
    X.A0 = dynres.dyn_g_0;
else
    X.A0 = zeros(m,1);
end

% construction of higher-order terms
state_index = [ones(n,1); zeros(k,1)];
shock_index = [zeros(n,1); ones(k,1)];

while (~iter_finish)
    if (isfield(dynres,sprintf('dyn_g_%d',model.expansion_order+1)))
        % field exists, expand
        model.expansion_order = model.expansion_order + 1;
        % matrix to be decomposed
        dynA = dynres.(sprintf('dyn_g_%d',model.expansion_order));
        % dynA needs to be split into subcomponents. For instance,
        % for order = 2, these are AXX AXW AWW.
        dynAfull = zeros(size(dynA,1), (n+k)^model.expansion_order);
        
        i_max = 0;
        i_cur = zeros(repmat((n+k),1,model.expansion_order));

        for i = 1 : (n+k)^model.expansion_order

            i_decomp = zeros(1,model.expansion_order);
            i_resid = i-1;
            j = 1;
            while (i_resid > 0)
                i_decomp(j) = mod(i_resid, n+k);
                i_resid = (i_resid - i_decomp(j)) / (n+k);
                j = j + 1;
            end
            i_decomp = sort(i_decomp);
            i_new = sum(i_decomp.*((n+k).^[model.expansion_order-1:-1:0]))+1;
            if (i_cur(i_new) > 0)
                % index already repeated
                i_select = i_cur(i_new);
            else
                % new index
                i_max = i_max+1;
                i_cur(i_new) = i_max;
                i_select = i_max;
            end
            
%             disp(sprintf('i %d    imax %d   i_cur %d    idecomp below',i, i_max, i_cur(i_new)))
%             disp(i_decomp+1);
            dynAfull(:,i) = dynA(:,i_cur(i_new));
        end
        
        X.(sprintf('A%d',model.expansion_order)) = dynAfull;
        
        % now decompose dynAfull into X and W components
        for i = 0 : model.expansion_order
            varname = '';
            Wkron = 1;
            for j = 1 : i
                Wkron = kron(Wkron,shock_index);
                varname = strcat('W',varname);
            end
            Xkron = 1;
            for j = 1 : model.expansion_order - i
                Xkron = kron(Xkron,state_index);
                varname = strcat('X',varname);
            end
            XWkron = kron(Xkron,Wkron);
            XWkron = (XWkron == 1);
            % these quantities are scaled by 1 / order!, where order is the
            % current order of expansion
            X.(sprintf('A%s',varname)) = dynAfull(:,XWkron);        
            
        end
        
    else
        iter_finish = 1;
        if (param.env_displayverbose)
            disp(sprintf('Expansion coefficients up to order %d constructed.', ...
                model.expansion_order));
        end
    end

end

% if expansion order is only 1, generate zero second-order matrices
if (model.expansion_order < 2)
    X.A2 = zeros(m,(n+k)^2);
    X.AXX = zeros(m,n*n);
    X.AXW = zeros(m,n*k);
    X.AWW = zeros(m,k*k);
end

% deterministic steady state
X.XSTEADY = dynres.dyn_steady_states(:,1);
% point of expansion (i.e. the point relative to which the expansion is
% expressed). This is the deterministic steady state when --no-centralize
% is used, or the fix point of the rule without the --no-centralize rule.
X.XEXP = dynres.dyn_ss;
% mean of the stationary distribution
X.XMEAN = dynres.dyn_mean;
% variance matrix of the shocks
X.SIGMA2 = dynres.dyn_vcov_exo; 

model.X = X;

model = processfile_dynare_step2(model,param);

end