function model = recursive_dynamics(model,param)
% Produces a triangular, recursively linear approximation of the model
% dynamics. Two equivalent representations are produced, depending on the
% value of the model_recursive parameter:
% 1 - splits dynamics into X1 and X2 (respects the separation of first- and
%     second-order dynamics).
% 2 - splits dynamics into X1 and X1+X2 (X1+X2 is easier to interpret as
%     the original dynamics of X).

if (param.model_recursive == 1)
    model = recursive_dynamics_version1(model,param);
elseif (param.model_recursive == 2)
    model = recursive_dynamics_version2(model,param);
end

end

function model = recursive_dynamics_version1(model,param)
% Produces a recursively-linear model solution. First n rows is X1,
% second n rows is X2, and then all non-state variables follow,
% as functions of X1 and X2.

% =========================================================================
% distribute model structure
fn = fieldnames(model);
for i = 1 : length(fn)
    eval(sprintf('%s = model.(cell2mat(fn(i)));',cell2mat(fn(i))));
end

% =========================================================================
disp_reg(param,0,sprintf('Creating first- and second-order dynamics (X1 and X2).'));

m = size(X.AX,1); % number of all variables
n = size(X.AX,2); % length of the actual state vector
k = size(X.AW,2); % length of the shock vector

nonstate_vars = setdiff([1:m],state_vars);
XEXPnew = [ X.XEXP(state_vars); X.XEXP(state_vars); X.XEXP(nonstate_vars) ];
XSTEADYnew = [ X.XSTEADY(state_vars); X.XSTEADY(state_vars); X.XSTEADY(nonstate_vars) ];
% XHMEANnew = [ X.XHMEAN(state_vars); zeros(length(state_vars),1); X.XHMEAN(nonstate_vars) ];
XMEANnew = [ X.XMEAN(state_vars); X.XMEAN(state_vars); X.XMEAN(nonstate_vars) ];

A0new = [ zeros(n,1); X.A0(state_vars); X.A0(nonstate_vars) ];
A1new = [ X.A1(state_vars,1:n) zeros(n,n) X.A1(state_vars,n+1:n+k);
          zeros(n,n) X.A1(state_vars,1:n) zeros(n,k);
          X.A1(nonstate_vars,1:n) X.A1(nonstate_vars,1:n) X.A1(nonstate_vars,n+1:n+k)];
A2part1 = zeros(n,(2*n+k)*(2*n+k));
A2part2 = zeros(m-n,(2*n+k)*(2*n+k));
for i = 1 : n
    a = X.A2(state_vars(i),:);
    mata = mat(a,n+k,n+k);
    mata = 1/2*(mata+mata');
    mataa = [mata(1:n,1:n) zeros(n,n) mata(1:n,n+1:n+k);
             zeros(n,2*n+k);
             mata(n+1:n+k,1:n) zeros(k,n) mata(n+1:n+k,n+1:n+k)];
%     mataa = [zeros(n,2*n+k); zeros(n+k,n) mata];
    A2part1(i,:) = transpose(mataa(:));
end
for i = 1 : m-n
    a = X.A2(nonstate_vars(i),:);
    mata = mat(a,n+k,n+k);
    mata = 1/2*(mata+mata');
    mataa = [mata(1:n,1:n) zeros(n,n) mata(1:n,n+1:n+k);
             zeros(n,2*n+k);
             mata(n+1:n+k,1:n) zeros(k,n) mata(n+1:n+k,n+1:n+k)];
%     mataa = [zeros(n,2*n+k); zeros(n+k,n) mata];
    A2part2(i,:) = transpose(mataa(:));
end
A2new = [zeros(n,(2*n+k)^2); A2part1; A2part2];

% X.XEXPold = X.XEXP;
% X.A0old = X.A0;
% X.A1old = X.A1;
% X.A2old = X.A2;

X.XEXP = XEXPnew;
X.XSTEADY = XSTEADYnew;
X.XMEAN = XMEANnew;
X.A0 = A0new;
X.A1 = A1new;
X.A2 = A2new;

% map new variables into their old positions
model.var_index_orig = [state_vars'; state_vars'; nonstate_vars'];
% rewrite previously saved indices
state_vars_old = state_vars;
state_vars = [1:2*n];
state_vars_first = [1:n];
state_vars_second = [n+1:2*n];

% state_vars = param.state_vars;

% create a new list of variable names
newlist = repmat(' ',m+n,size(var_names,2)+7);
newlist(1:n,1:size(var_names,2)) = var_names(state_vars_old,:);
newlist(2*n+1:n+m,1:size(var_names,2)) = var_names(nonstate_vars,:);
newlist_tex = repmat(' ',m+n,size(var_names_tex,2)+13);
newlist_tex(1:n,1:size(var_names_tex,2)) = var_names_tex(state_vars_old,:);
newlist_tex(2*n+1:n+m,1:size(var_names_tex,2)) = var_names_tex(nonstate_vars,:);
for i = 1 : n
    st = strcat(strtrim(var_names(state_vars_old(i),:)),'-first');
    newlist(i,1:length(st)) = st;
    st = strcat(strtrim(var_names_tex(state_vars_old(i),:)),' first-order');
    newlist_tex(i,1:length(st)) = st;
end
for i = 1 : n
    st = strcat(strtrim(var_names(state_vars_old(i),:)),'-second');
    newlist(n+i,1:length(st)) = st;
    st = strcat(strtrim(var_names_tex(state_vars_old(i),:)),' second-order');
    newlist_tex(n+i,1:length(st)) = st;
end

model.var_names = newlist;
model.var_names_tex = newlist_tex;
model.state_vars = state_vars;
model.state_vars_first = state_vars_first;
model.state_vars_second = state_vars_second;


m = m + n;
n = 2*n;

% now decompose dynAfull AGAIN into X and W components

state_index = [ones(n,1); zeros(k,1)];
shock_index = [zeros(n,1); ones(k,1)];

for ii = 1 : 2 % model.expansion_order
    for i = 0 : ii
        varname = '';
        Wkron = 1;
        for j = 1 : i
            Wkron = kron(Wkron,shock_index);
            varname = strcat('W',varname);
        end
        Xkron = 1;
        for j = 1 : ii - i
            Xkron = kron(Xkron,state_index);
            varname = strcat('X',varname);
        end
        XWkron = kron(Xkron,Wkron);
        XWkron = (XWkron == 1);
        % these quantities are scaled by 1 / order!, where order is the
        % current order of expansion
        dynAfull = X.(sprintf('A%d',ii));
        X.(sprintf('A%s',varname)) = dynAfull(:,XWkron);        

    end
end

model.X = X;

end

%% ========================================================================

function model = recursive_dynamics_version2(model,param)
% Produces a recursively-linear model solution. First n rows is X1,
% second n rows is X1+X2, and then all non-state variables follow,
% as functions of X1 and X1+X2.

% =========================================================================
% distribute model structure
fn = fieldnames(model);
for i = 1 : length(fn)
    eval(sprintf('%s = model.(cell2mat(fn(i)));',cell2mat(fn(i))));
end

% =========================================================================
disp_reg(param,0,sprintf('Creating first- and second-order dynamics (X1 and X1+X2).'));

m = size(X.AX,1); % number of all variables
n = size(X.AX,2); % length of the actual state vector
k = size(X.AW,2); % length of the shock vector

nonstate_vars = setdiff([1:m],state_vars);
XEXPnew = [ X.XEXP(state_vars); X.XEXP(state_vars); X.XEXP(nonstate_vars) ];
XSTEADYnew = [ X.XSTEADY(state_vars); X.XSTEADY(state_vars); X.XSTEADY(nonstate_vars) ];
% XHMEANnew = [ X.XHMEAN(state_vars); zeros(length(state_vars),1); X.XHMEAN(nonstate_vars) ];
XMEANnew = [ X.XMEAN(state_vars); X.XMEAN(state_vars); X.XMEAN(nonstate_vars) ];

A0new = [ zeros(n,1); X.A0(state_vars); X.A0(nonstate_vars) ];
A1new = [ X.A1(state_vars,1:n) zeros(n,n) X.A1(state_vars,n+1:n+k);
          zeros(n,n) X.A1(state_vars,1:n) X.A1(state_vars,n+1:n+k);
          zeros(m-n,n) X.A1(nonstate_vars,1:n) X.A1(nonstate_vars,n+1:n+k)];
A2part1 = zeros(n,(2*n+k)*(2*n+k));
A2part2 = zeros(m-n,(2*n+k)*(2*n+k));
for i = 1 : n
    a = X.A2(state_vars(i),:);
    mata = mat(a,n+k,n+k);
    mata = 1/2*(mata+mata');
    mataa = [mata(1:n,1:n) zeros(n,n) mata(1:n,n+1:n+k);
             zeros(n,2*n+k);
             mata(n+1:n+k,1:n) zeros(k,n) mata(n+1:n+k,n+1:n+k)];
%     mataa = [zeros(n,2*n+k); zeros(n+k,n) mata];
    A2part1(i,:) = transpose(mataa(:));
end
for i = 1 : m-n
    a = X.A2(nonstate_vars(i),:);
    mata = mat(a,n+k,n+k);
    mata = 1/2*(mata+mata');
    mataa = [mata(1:n,1:n) zeros(n,n) mata(1:n,n+1:n+k);
             zeros(n,2*n+k);
             mata(n+1:n+k,1:n) zeros(k,n) mata(n+1:n+k,n+1:n+k)];
%     mataa = [zeros(n,2*n+k); zeros(n+k,n) mata];
    A2part2(i,:) = transpose(mataa(:));
end
A2new = [zeros(n,(2*n+k)^2); A2part1; A2part2];

% X.XEXPold = X.XEXP;
% X.A0old = X.A0;
% X.A1old = X.A1;
% X.A2old = X.A2;

X.XEXP = XEXPnew;
X.XSTEADY = XSTEADYnew;
X.XMEAN = XMEANnew;
X.A0 = A0new;
X.A1 = A1new;
X.A2 = A2new;

% map new variables into their old positions
model.var_index_orig = [state_vars'; state_vars'; nonstate_vars'];
% rewrite previously saved indices
state_vars_old = state_vars;
state_vars = [1:2*n];
state_vars_first = [1:n];
state_vars_second = [n+1:2*n];

% state_vars = param.state_vars;

% create a new list of variable names
newlist = repmat(' ',m+n,size(var_names,2)+6);
newlist(1:n,1:size(var_names,2)) = var_names(state_vars_old,:);
newlist(2*n+1:n+m,1:size(var_names,2)) = var_names(nonstate_vars,:);
newlist_tex = repmat(' ',m+n,size(var_names_tex,2)+12);
newlist_tex(1:n,1:size(var_names_tex,2)) = var_names_tex(state_vars_old,:);
newlist_tex(2*n+1:n+m,1:size(var_names_tex,2)) = var_names_tex(nonstate_vars,:);
for i = 1 : n
    st = strcat(strtrim(var_names(state_vars_old(i),:)),'-first');
    newlist(n+i,1:length(st)) = st;
    st = strcat(strtrim(var_names_tex(state_vars_old(i),:)),' first-order');
    newlist_tex(n+i,1:length(st)) = st;
end

model.var_names = newlist;
model.var_names_tex = newlist_tex;
model.state_vars = state_vars;
model.state_vars_first = state_vars_first;
model.state_vars_second = state_vars_second;

m = m + n;
n = 2*n;

% now decompose dynAfull AGAIN into X and W components

state_index = [ones(n,1); zeros(k,1)];
shock_index = [zeros(n,1); ones(k,1)];

for ii = 1 : min(model.expansion_order,2)
    for i = 0 : ii
        varname = '';
        Wkron = 1;
        for j = 1 : i
            Wkron = kron(Wkron,shock_index);
            varname = strcat('W',varname);
        end
        Xkron = 1;
        for j = 1 : ii - i
            Xkron = kron(Xkron,state_index);
            varname = strcat('X',varname);
        end
        XWkron = kron(Xkron,Wkron);
        XWkron = (XWkron == 1);
        % these quantities are scaled by 1 / order!, where order is the
        % current order of expansion
        dynAfull = X.(sprintf('A%d',ii));
        X.(sprintf('A%s',varname)) = dynAfull(:,XWkron);        

    end
end

model.X = X;
    
end